package javax.management.remote.rmi;

import java.io.ObjectStreamClass;
import java.lang.reflect.Method;
import java.util.List;
import sun.rmi.server.DeserializationChecker;
import javax.security.auth.Subject;
import java.rmi.server.RemoteObject;
import java.rmi.NoSuchObjectException;
import java.rmi.RemoteException;
import sun.rmi.server.UnicastServerRef;
import sun.rmi.server.UnicastServerRef2;
import java.rmi.server.UnicastRemoteObject;
import com.sun.jmx.remote.util.EnvHelp;
import com.sun.jmx.remote.internal.RMIExporter;
import java.rmi.Remote;
import java.io.IOException;
import sun.reflect.misc.ReflectUtil;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Map;
import java.rmi.server.RMIServerSocketFactory;
import java.rmi.server.RMIClientSocketFactory;

public class RMIJRMPServerImpl extends RMIServerImpl
{
    private final RMIJRMPServerImpl.ExportedWrapper exportedWrapper;
    private final int port;
    private final RMIClientSocketFactory csf;
    private final RMIServerSocketFactory ssf;
    private final Map<String, ?> env;
    
    public RMIJRMPServerImpl(int n, RMIClientSocketFactory csf, RMIServerSocketFactory ssf, Map<String, ?> map) throws IOException {
        super(map);
        if (n < 0) {
            throw new IllegalArgumentException("Negative port: " + n);
        }
        this.port = n;
        this.csf = csf;
        this.ssf = ssf;
        this.env = ((map == null) ? Collections.emptyMap() : map);
        String[] array = (String[])(String[])this.env.get("jmx.remote.rmi.server.credential.types");
        ArrayList<String> list = null;
        if (array != null) {
            list = new ArrayList();
            int length = array.length;
            int i = 0;
            while (i < length) {
                String name = array[i];
                if (name == null) {
                    throw new IllegalArgumentException("A credential type is null.");
                }
                ReflectUtil.checkPackageAccess(name);
                list.add(name);
                ++i;
            }
        }
        this.exportedWrapper = ((list != null) ? new RMIJRMPServerImpl.ExportedWrapper(this, list) : null);
        return;
    }
    
    @Override
    protected void export() throws IOException {
        if (this.exportedWrapper != null) {
            this.export(this.exportedWrapper);
        }
        else {
            this.export(this);
        }
    }
    
    private void export(Remote impl) throws RemoteException {
        RMIExporter rmiExporter = (RMIExporter)this.env.get("com.sun.jmx.remote.rmi.exporter");
        int serverDaemon = EnvHelp.isServerDaemon(this.env);
        if (serverDaemon && rmiExporter != null) {
            throw new IllegalArgumentException("If jmx.remote.x.daemon is specified as true, com.sun.jmx.remote.rmi.exporter cannot be used to specify an exporter!");
        }
        if (!(serverDaemon == 0)) {
            if (this.csf == null && this.ssf == null) {
                new UnicastServerRef(this.port).exportObject(impl, null, true);
            }
            else {
                new UnicastServerRef2(this.port, this.csf, this.ssf).exportObject(impl, null, true);
            }
        }
        else if (rmiExporter != null) {
            rmiExporter.exportObject(impl, this.port, this.csf, this.ssf);
        }
        else {
            UnicastRemoteObject.exportObject(impl, this.port, this.csf, this.ssf);
        }
        return;
    }
    
    private void unexport(Remote obj, boolean force) throws NoSuchObjectException {
        RMIExporter rmiExporter = (RMIExporter)this.env.get("com.sun.jmx.remote.rmi.exporter");
        if (rmiExporter == null) {
            UnicastRemoteObject.unexportObject(obj, force);
        }
        else {
            rmiExporter.unexportObject(obj, force);
        }
    }
    
    @Override
    protected String getProtocol() {
        return "rmi";
    }
    
    @Override
    public Remote toStub() throws IOException {
        if (this.exportedWrapper != null) {
            return RemoteObject.toStub(this.exportedWrapper);
        }
        return RemoteObject.toStub(this);
    }
    
    @Override
    protected RMIConnection makeClient(String s, Subject subject) throws IOException {
        if (s == null) {
            throw new NullPointerException("Null connectionId");
        }
        RMIConnectionImpl rmiConnectionImpl = new RMIConnectionImpl(this, s, this.getDefaultClassLoader(), subject, this.env);
        this.export(rmiConnectionImpl);
        return rmiConnectionImpl;
    }
    
    @Override
    protected void closeClient(RMIConnection rmiConnection) throws IOException {
        this.unexport(rmiConnection, true);
    }
    
    @Override
    protected void closeServer() throws IOException {
        if (this.exportedWrapper != null) {
            this.unexport(this.exportedWrapper, true);
        }
        else {
            this.unexport(this, true);
        }
    }
    
    private static class ExportedWrapper implements RMIServer, DeserializationChecker
    {
        private final RMIServer impl;
        private final List<String> allowedTypes;
        
        private ExportedWrapper(RMIServer impl, List<String> allowedTypes) {
            this.impl = impl;
            this.allowedTypes = allowedTypes;
        }
        
        @Override
        public String getVersion() throws RemoteException {
            return this.impl.getVersion();
        }
        
        @Override
        public RMIConnection newClient(Object o) throws IOException {
            return this.impl.newClient(o);
        }
        
        @Override
        public void check(Method method, ObjectStreamClass objectStreamClass, int n, int n2) {
            String name = objectStreamClass.getName();
            if (!this.allowedTypes.contains(name)) {
                throw new ClassCastException("Unsupported type: " + name);
            }
            return;
        }
        
        @Override
        public void checkProxyClass(Method method, String[] array, int n, int n2) {
            if (array != null && array.length > 0) {
                int length = array.length;
                int i = 0;
                while (i < length) {
                    String str = array[i];
                    if (!this.allowedTypes.contains(str)) {
                        throw new ClassCastException("Unsupported type: " + str);
                    }
                    ++i;
                }
            }
        }
    }
}
